//
//  Student.m
//  Lab1d
//
//  Created by R.J. Aquino on 7/7/13.
//  Copyright (c) 2013 CS50. All rights reserved.
//

#import "Student.h"

@implementation Student

/*
 * Init method
 */
- (Student *)init {
    self = [super init];
    
    // initialize major to undeclared
    self.major = @"Undeclared";
    
    // initialize NSMutableDictionary for grades
    self.grades = [[NSMutableDictionary alloc] init];
    return self;
}

/*
 * Legacy name method. Returns full name.
 */
- (NSString*)name {
    return [NSString stringWithFormat:@"%@ %@", self.first_name, self.last_name];
}

/*
 * Overwritten setMajor to enfoce valid majors only.
 */
- (void)setMajor:(NSString *)major {
    // get valid majors
    NSArray *majors = [Student validMajors];
    
    // check if requested major is valid
    if ([majors containsObject:major])
        _major = [major copy];
    else
        NSLog(@"%@ is not a valid major. %@'s major is still %@", major, self.name, self.major);
}

/*
 * Returns the valid majors
 */
+ (NSArray *)validMajors {
    NSArray *majors = [[NSArray alloc] initWithObjects:@"Math", @"English", @"Computer Science", @"Undeclared",  nil];
    return majors;
}

/*
 * Over written description method (the description method of an NSObject is secretly what's called when you print or NSLog "%@".
 */
- (NSString*)description {
    // get last two digits from year
    int trimmed_year = self.year % 1000 % 100;
    
    // return string (%02d means that years like 2004 will come out 04, up to two digits, padded with zeroes)
    return [NSString stringWithFormat:@"%@ '%02d, %@", self.name, trimmed_year, self.major];
}

/*
 * Legacy method for age, using birthday property
 */
- (int)age {
    
    // use the NSCalendar methods for date math
    // see http://stackoverflow.com/questions/4739483/number-of-days-between-two-nsdates  and
    // http://cocoamatic.blogspot.com/2010/09/nsdate-number-of-days-between-two-dates.html for more details
    NSCalendar *calendar = [NSCalendar currentCalendar];
    NSUInteger unitFlags = NSYearCalendarUnit;
    NSDateComponents *components = [calendar components:unitFlags fromDate:self.birthday toDate:[NSDate date] options:0];
    
    
    return (int)[components year];
    
    
}

/*
 * Instance method for adding a grade. We could do this directly in main.m, but if we ever changed
 * how grades were stored, we'd have to change every line that added a grade.
 */
- (void)addGradeForClass:(int)classNum WithGrade:(int)grade {
    [self.grades setValue:[NSNumber numberWithInt:grade] forKey:[NSString stringWithFormat:@"%d", classNum]];
}

/*
 * Returns a line-separated string of a student's grades.
 */
- (NSString*)transcript {
    // if dictionary is empty, report so
    if ([self.grades count] == 0)
        return @"No grades.";
    
    // otherwise, build up string of grades
    NSString *result = @"";
    for (NSString *class in self.grades) {
        
        // get NSNumber from dictionary
        NSNumber* gradeNum = [self.grades objectForKey:class];
        
        // convert number to integer, for printing as %d (not neccesary, but nice to show)
        int gradeInt = [gradeNum intValue];
        
        // build string to add
        NSString *stringToAdd = [NSString stringWithFormat:@"%@: %d\n", class, gradeInt];
        
        // append string
        result = [result stringByAppendingString:stringToAdd];

        // note: this could all be done in one line as follows: result = [result stringByAppendingString:[NSString stringWithFormat:@"%@: %d\n", class, [(NSNumber*)[self.grades objectForKey:class] intValue]]];
    }
    
    return result;
}

/*
 * Returns the average of a student's grades.
 */
- (float)average {
    
    int numGrades = [self.grades count];
    if (numGrades == 0)
        return 0;
    
    // iterate over all grades
    float total = 0;
    for (NSString *class in self.grades) {
        
        // get NSNumber from dictionary
        NSNumber* gradeNum = [self.grades objectForKey:class];
        
        // convert number to integer
        int gradeInt = [gradeNum intValue];
        
        // add to total
        total += gradeInt;
    }
    
    return total/numGrades;
}
@end
